@@ -3,8 +3,6 @@
 #include "DisplayTerminal.hh"
 #include "Symbols.hh"
 #include "properties/Accent.hh"
-#include "NTensor.hh"
-#include <variant>
 
 using namespace cadabra;
 
@@ -156,28 +154,63 @@ void DisplayTerminal::print_children(std::ostream& str, Ex::iterator it, int ski
 
 void DisplayTerminal::print_multiplier(std::ostream& str, Ex::iterator it, int mult)
 	{
-	if(it->multiplier->is_rational()) {
-		mpq_class q = it->multiplier->get_rational();
-		mpz_class denom=q.get_den();
-		mpz_class numer=q.get_num();
-		
-		if(denom!=1) {
-			if(mult*numer<0) {
-				str << " - ";
-				mult *= -1;
-				}
-			str << " " << mult * numer << "/" << denom << " ";
-			}
-		else if(mult * it->multiplier->get_rational()==-1) {
-			str << "-";
-			}
-		else {
-			str << mult * (it->multiplier->get_rational());
+	mpz_class denom=it->multiplier->get_den();
+
+	if(denom!=1) {
+		if(mult*it->multiplier->get_num()<0) {
+			str << " - ";
+			mult *= -1;
 			}
+		str << " " << mult * it->multiplier->get_num() << "/" << it->multiplier->get_den() << " ";
+		}
+	else if(mult * (*it->multiplier)==-1) {
+		str << "-";
 		}
 	else {
-		str << *it->multiplier;
+		str << mult * (*it->multiplier);
 		}
+
+	/*	bool turned_one=false;
+		mpz_class denom=it->multiplier->get_den();
+
+		if(*it->multiplier<0) {
+			if(*tree.parent(it)->name=="\\sum") { // sum takes care of minus sign
+				if(*it->multiplier!=-1) {
+					if(denom!=1) {
+						str << "\\frac{" << -(it->multiplier->get_num()) << "}{"
+						 << it->multiplier->get_den() << "}";
+								}
+					else {
+						str << -(*it->multiplier);
+								}
+							}
+				else                    turned_one=true;
+						}
+			else	{
+				if(denom!=1) {
+					str << "(\\frac{" << it->multiplier->get_num() << "}{"
+					 << it->multiplier->get_den() << "})";
+							}
+				else if(*it->multiplier==-1) {
+					str << "-";
+				turned_one=true;
+							}
+				else {
+					str << "(" << *it->multiplier << ")";
+							}
+						}
+					}
+		else {
+			if(denom!=1) {
+				str << "\\frac{" << it->multiplier->get_num() << "}{"
+				 << it->multiplier->get_den() << "}";
+						}
+			else
+				str << *it->multiplier;
+					}
+
+		if(!turned_one && !(*it->name=="1"))
+		str << "*"; */
 	}
 
 void DisplayTerminal::print_opening_bracket(std::ostream& str, str_node::bracket_t br, str_node::parent_rel_t pr)
@@ -256,21 +289,6 @@ void DisplayTerminal::print_parent_rel(std::ostream& str, str_node::parent_rel_t
 
 void DisplayTerminal::dispatch(std::ostream& str, Ex::iterator it)
 	{
-	if(handle_unprintable_wildcards(str, it))
-		return;
-	
-	if(std::holds_alternative<std::shared_ptr<NTensor>>(it->content)) {
-		auto nt = std::get<std::shared_ptr<NTensor>>(it->content);
-		str << (*nt);
-//		std::string s = str2.str();
-//		str << std::regex_replace(s, std::regex("(\\d+\\.?\\d*|\\.\\d+)e([+-]?\\d+)"), "$1 \\times 10^{$2}");
-		return; // FIXME: always just ignore the subtree?
-		}
-	else if(std::holds_alternative<std::shared_ptr<NInterpolatingFunction>>(it->content)) {
-		str << " (NInterpolatingFunction)";
-		return;
-		}
-	
 	if(*it->name=="\\prod")            print_productlike(str, it, " ");
 	else if(*it->name=="\\sum")        print_sumlike(str, it);
 	else if(*it->name=="\\frac")       print_fraclike(str, it);
@@ -291,20 +309,6 @@ void DisplayTerminal::dispatch(std::ostream& str, Ex::iterator it)
 	else                               print_other(str, it);
 	}
 
-bool DisplayTerminal::handle_unprintable_wildcards(std::ostream& str, Ex::iterator it) const
-	{
-	// Catch `\pow{#}` and other wildcard constructions, these
-	// need to print verbatim.
-
-	if(it.number_of_children()==1) {
-		if(*(it.begin()->name)=="#") {
-			str << (*(it->name)) << "{#}";
-			return true;
-			}
-		}
-	return false;
-	}
-
 void DisplayTerminal::print_commalike(std::ostream& str, Ex::iterator it)
 	{
 	Ex::sibling_iterator sib=tree.begin(it);
@@ -605,18 +609,16 @@ void DisplayTerminal::print_other(std::ostream& str, Ex::iterator it)
 
 	if(needs_extra_brackets) str << "{"; // to prevent double sup/sub script errors
 	std::string sbit=*it->name;
-	if(sbit!="\\indexbracket") {
-		if(use_unicode && getenv("CADABRA_NO_UNICODE")==0) {
-			auto rn1 = symmap.find(sbit);
-			if(rn1!=symmap.end())
-				sbit = rn1->second;
-			auto rn = cadabra::symbols::greekmap.find(sbit);
-			if(rn!=cadabra::symbols::greekmap.end())
-				sbit = rn->second;
-			}
-		str << sbit;
+	if(use_unicode && getenv("CADABRA_NO_UNICODE")==0) {
+		auto rn1 = symmap.find(sbit);
+		if(rn1!=symmap.end())
+			sbit = rn1->second;
+		auto rn = cadabra::symbols::greekmap.find(sbit);
+		if(rn!=cadabra::symbols::greekmap.end())
+			sbit = rn->second;
 		}
-	
+	str << sbit;
+
 	if(needs_extra_brackets) str << "}";
 
 	print_children(str, it);
