@@ -314,15 +314,10 @@ pybind11::object ExNode::get_multiplier() const
 	if(!ex->is_valid(it))
 		throw ConsistencyException("Cannot get the multiplier of an iterator before the first 'next'.");
 
-	if(it->multiplier->is_rational()) {
-		pybind11::object mpq = pybind11::module::import("gmpy2").attr("mpq");
-		auto m = it->multiplier->get_rational();
-		pybind11::object mult = mpq(m.get_num().get_si(), m.get_den().get_si());
-		return mult;
-		}
-	else {
-		return pybind11::cast(it->multiplier->get_double());
-		}
+	pybind11::object mpq = pybind11::module::import("gmpy2").attr("mpq");
+	auto m = *it->multiplier;
+	pybind11::object mult = mpq(m.get_num().get_si(), m.get_den().get_si());
+	return mult;
 	}
 
 void ExNode::set_multiplier(pybind11::object mult)
@@ -462,7 +457,7 @@ std::string ExNode::__str__() const
 	// //		str << "(unchanged)" << std::endl;
 	// 	DisplayTeX dt(get_kernel_from_scope()->properties, ex);
 
-	DisplayTerminal dt(*get_kernel_from_scope(), Ex(it), true);
+	DisplayTerminal dt(*get_kernel_from_scope(), it, true);
 	dt.output(str, it);
 
 	return str.str();
@@ -474,7 +469,7 @@ std::string ExNode::_latex_() const
 		throw ConsistencyException("Cannot print iterator LaTeX form before the first 'next'.");
 
 	std::ostringstream str;
-	DisplayTeX dt(*get_kernel_from_scope(), Ex(it));
+	DisplayTeX dt(*get_kernel_from_scope(), it);
 	dt.output(str, it);
 	return str.str();
 	}
